@@ -150,6 +150,8 @@ int netsnmp_mem_arch_load( netsnmp_cache *cache, void *magic ) {
         mem->other = -1;
     }
 
+   /* Shared memory is not reported by Linux 2.6 kernel */
+   if (0 != netsnmp_os_prematch("Linux","2.6")) {
     mem = netsnmp_memory_get_byIdx( NETSNMP_MEM_TYPE_SHARED, 1 );
     if (!mem) {
         snmp_log_perror("No Shared Memory info entry");
@@ -161,6 +163,7 @@ int netsnmp_mem_arch_load( netsnmp_cache *cache, void *magic ) {
         mem->free  = -1;
         mem->other = -1;
     }
+   }
 
     mem = netsnmp_memory_get_byIdx( NETSNMP_MEM_TYPE_CACHED, 1 );
     if (!mem) {
@@ -170,7 +173,7 @@ int netsnmp_mem_arch_load( netsnmp_cache *cache, void *magic ) {
              mem->descr = strdup("Cached memory");
         mem->units = 1024;
         mem->size  = cached;
-        mem->free  = -1;
+        mem->free  = 0;     /* Report cached size/used as equal */
         mem->other = -1;
     }
 
@@ -193,8 +196,10 @@ int netsnmp_mem_arch_load( netsnmp_cache *cache, void *magic ) {
         if (!mem->descr)
              mem->descr = strdup("Memory buffers");
         mem->units = 1024;
-        mem->size  = buffers;
-        mem->free  = -1;
+        mem->size  = memtotal;  /* Traditionally we've always regarded
+                                   all memory as potentially available
+                                   for memory buffers. */
+        mem->free  = memtotal - buffers;
         mem->other = -1;
     }
 
