@@ -7,7 +7,7 @@
 #     modify it under the same terms as Perl itself.
 
 package SNMP;
-$VERSION = '5.04011';   # current release version number
+$VERSION = '5.0402';   # current release version number
 
 require Exporter;
 require DynaLoader;
@@ -685,7 +685,7 @@ sub gettable {
 	    if ($parse_indexes) {
 		# get indexes
 		my @indexes =
-		  @{$SNMP::MIB{$textnode}{'children'}[0]{'indexes'}};
+		  @{$SNMP::MIB{$textnode}{'children'}[0]{'indexes'} || [] };
 		# quick translate into a hash
 		map { $indexes{$_} = 1; } @indexes;
 	    }
@@ -701,7 +701,8 @@ sub gettable {
 		# some tables are only indexes, and we need to walk at
 		# least one column.  We pick the last.
 		push @{$state->{'columns'}}, $root_oid . ".1." .
-		  $children->[$#$children]{'subID'};
+		  $children->[$#$children]{'subID'}
+		  if ref($state) eq 'HASH' and ref($children) eq 'HASH';
 	    }
 	}
     } else {
@@ -731,7 +732,7 @@ sub gettable {
     $vbl = $state->{'varbinds'};
 	
     my $repeatcount;
-    if ($this->{Version} == 1 || $state->{'options'}{nogetbulk}) {
+    if ($this->{Version} eq '1' || $state->{'options'}{nogetbulk}) {
 	$state->{'repeatcount'} = 1;
     } elsif ($state->{'options'}{'repeat'}) {
 	$state->{'repeatcount'} = $state->{'options'}{'repeat'};
@@ -753,7 +754,7 @@ sub gettable {
     # call the next processing function ourself.
     #
     if ($state->{'options'}{'callback'}) {
-	if ($this->{Version} > 1 && !$state->{'options'}{'nogetbulk'}) {
+	if ($this->{Version} ne '1' && !$state->{'options'}{'nogetbulk'}) {
 	    $res = $this->getbulk(0, $state->{'repeatcount'}, $vbl,
 				  [\&_gettable_do_it, $this, $vbl,
 				   $parse_indexes, $textnode, $state]);
@@ -763,7 +764,7 @@ sub gettable {
 				   $parse_indexes, $textnode, $state]);
 	}
     } else {
-	if ($this->{Version} > 1 && !$state->{'options'}{'nogetbulk'}) {
+	if ($this->{Version} ne '1' && !$state->{'options'}{'nogetbulk'}) {
 	    $res = $this->getbulk(0, $state->{'repeatcount'}, $vbl);
 	} else {
 	    $res = $this->getnext($vbl);
@@ -845,7 +846,7 @@ sub _gettable_do_it() {
         # call the next processing function ourself.
         #
 	if ($state->{'options'}{'callback'}) {
-	    if ($this->{Version} > 1 && !$state->{'options'}{'nogetbulk'}) {
+	    if ($this->{Version} ne '1' && !$state->{'options'}{'nogetbulk'}) {
 		$res = $this->getbulk(0, $state->{'repeatcount'}, $vbl,
 				      [\&_gettable_do_it, $this, $vbl,
 				       $parse_indexes, $textnode, $state]);
@@ -856,7 +857,7 @@ sub _gettable_do_it() {
 	    }
 	    return;
 	} else {
-	    if ($this->{Version} > 1 && !$state->{'options'}{'nogetbulk'}) {
+	    if ($this->{Version} ne '1' && !$state->{'options'}{'nogetbulk'}) {
 		$res = $this->getbulk(0, $state->{'repeatcount'}, $vbl);
 	    } else {
 		$res = $this->getnext($vbl);
